package br.org.indt.nokiatrends.server;

import br.org.indt.nokiatrends.osruntime.OSRunner;
import br.org.indt.nokiatrends.server.files.BTFileFilter;

import darkmaps.TheFrame;

import java.awt.Container;
/*
import java.awt.Dimension;
import java.awt.Toolkit;
*/

import java.io.File;

import java.util.Arrays;
import java.util.Comparator;
import java.util.logging.Logger;

import javax.swing.JFrame;

public class BTDirectoryListener implements Runnable
{
    public static final Logger log       = Logger.getLogger(BTDirectoryListener.class.getName());
    public static final long ZERO_5_SECS = 500;
    public static final long ONE_SEC     = 1000;
    public static final String JPG_EXT   = "jpg";
    public static final String PNG_EXT   = "png";
    public static final String NO_EXT    = null;
    private String pattern;
    private String ext;
    private File fileDir;
    private BTFileFilter fileFilter;
    private File lastAddFile;
    private TheFrame thFrame             = null;
    private OSRunner osRunner            = null;

    public BTDirectoryListener(String fileDirectory, String pattern, String ext, String iePath)
        throws BTDirectoryListenerException
    {
        this.fileDir = new File(fileDirectory);

        if (!fileDir.isDirectory())
        {
            throw new BTDirectoryListenerException("URI file string is not a directory!");
        }

        this.pattern        = pattern;
        this.ext            = ext;
        this.fileFilter     = new BTFileFilter(pattern, ext);
        this.osRunner       = new OSRunner(iePath);
    }

    /**
     * List all files in this directory, using BTFileFilter
     */
    public File[] listFiles()
    {
        return this.fileDir.listFiles(this.fileFilter);
    }

    /**
     * List all files in this directory (not using BTFileFilter)
     */
    public File[] listFilesNoFilter()
    {
        return this.fileDir.listFiles();
    }

    /**
     * List all files and dirs in this directory (not using BTFileFilter)
     */
    public String[] listNoFilter()
    {
        return this.fileDir.list();
    }

    public File getLastFile()
    {
        File[] files = this.fileDir.listFiles(this.fileFilter);

        if ((files != null) && (files.length > 0))
        {
        	/*
            // sorts the File's array, by the file name        	
            Arrays.sort(files,
                new Comparator()
                {
                    public int compare(Object obj1, Object obj2)
                    {
                        File file1 = (File) obj1;
                        File file2 = (File) obj2;

                        // remove file extensions
                        String fn1 = file1.getName().substring(0, file1.getName().lastIndexOf('.'));
                        String fn2 = file2.getName().substring(0, file2.getName().lastIndexOf('.'));

                        String cmp1;
                        String cmp2;

                        if (fn1.indexOf('(') != -1)
                        {
                            cmp1 = getImageNumber(fn1);
                        }
                        else
                        {
                            if (fn1.length() > pattern.length())
                            {
                                cmp1 = fn1.substring(fn1.indexOf(pattern) + pattern.length() + 1);
                            }
                            else
                            {
                                cmp1 = fn1;
                            }
                        }

                        if (fn2.indexOf('(') != -1)
                        {
                            cmp2 = getImageNumber(fn2);
                        }
                        else
                        {
                            if (fn2.length() > pattern.length())
                            {
                                cmp2 = fn2.substring(fn2.indexOf(pattern) + pattern.length() + 1);
                            }
                            else
                            {
                                cmp2 = fn2;
                            }
                        }

                        int nCmp1;
                        int nCmp2;

                        try
                        {
                            nCmp1     = Integer.parseInt(cmp1);
                            nCmp2     = Integer.parseInt(cmp2);

                            return (nCmp1 - nCmp2);
                        }
                        catch (NumberFormatException exc)
                        {
                            if (fn1.equalsIgnoreCase(fn2))
                            {
                                return 0;
                            }
                            else
                            {
                                return (fn1.compareToIgnoreCase(fn2));
                            }
                        }
                    }
                });
                */
        	Arrays.sort( files,  new Comparator() {
        		public int compare(Object obj1, Object obj2) {
        			File file1 = (File)obj1;
        			File file2 = (File)obj2;
        			if (file1.lastModified() == file2.lastModified())
        				return 0;
        			else if (file1.lastModified() - file2.lastModified()>0)
        				return 1;
        			else
        				return -1;
        		}
        	});

            return files[files.length - 1];
        }
        else
        {
            return null;
        }
    }

    private String getImageNumber(String imageName)
    {
        String number = null;
        int startNmbr = imageName.indexOf('(');
        int stopNmbr  = imageName.indexOf(')');

        if ((startNmbr >= 0) && (stopNmbr >= 0) && (startNmbr < stopNmbr))
        {
            number = imageName.substring(startNmbr + 1, stopNmbr);
        }

        //System.out.println("number - " + number);
        return number;
    }

    public JFrame createJFrame(Container zoom)
    {
        JFrame fr = new JFrame("Nokia Trends");
        fr.add(zoom);

        return fr;
    }

    public void run()
    {
        try
        {
            this.lastAddFile = getLastFile();
            log.info("Entering BT server thread...");

            while (true)
            {
                synchronized (this)
                {
                    File[] files = listFiles();
                	Arrays.sort( files,  new Comparator() {
                		public int compare(Object obj1, Object obj2) {
                			File file1 = (File)obj1;
                			File file2 = (File)obj2;
                			if (file1.lastModified() == file2.lastModified())
                				return 0;
                			else if (file1.lastModified() - file2.lastModified()>0)
                				return 1;
                			else
                				return -1;
                		}
                	});

                    if ((files != null) && (files.length > 0))
                    {
                        for (int i = 0; i < files.length; i++)
                        {
                            log.info("File " + i + ": " + files[i].getAbsolutePath());
                        }

                        File file = getLastFile();
                        System.out.println("Last added file ---> " + file.getName());

                        System.out.println("----------------");

                        if (lastAddFile != null)
                        {
                            System.out.println("lastAddFile = " + lastAddFile.getName());
                        }

                        String fnFile = null;

                        if (file != null)
                        {
                            System.out.println("file = " + file.getName());
                            System.out.println("----------------");

                            if (file.getName().indexOf('(') != -1)
                            {
                                fnFile = getImageNumber(file.getName());
                            }
                            else
                            {
                                fnFile = file.getName();
                            }
                        }

                        String fnLastFile = null;
                        boolean cmpStrs   = false;

                        if (lastAddFile != null)
                        {
                            if (lastAddFile.getName().indexOf('(') != -1)
                            {
                                fnLastFile = getImageNumber(lastAddFile.getName());
                            }
                            else
                            {
                                fnLastFile = lastAddFile.getName();
                            }
                        }

                        try
                        {
                            cmpStrs = Integer.parseInt(fnFile) > Integer.parseInt(fnLastFile);
                        }
                        catch (NumberFormatException exc)
                        {
                            cmpStrs = (fnFile.compareToIgnoreCase(fnLastFile) > 0);
                        }

                        // find if added file is newer than the older one
                        if ((fnFile != null) && (fnLastFile != null) && (file.lastModified()>lastAddFile.lastModified()))//&& cmpStrs)
                        {
                            this.lastAddFile = file;

                            if ((this.osRunner != null) && this.osRunner.isProcessRunning())
                            {
                                // Kills AppletViewer application
                                this.osRunner.killApp();
                            }

                            //this.osRunner.copyPhotoToCodeUp(this.lastAddFile.getAbsolutePath());
                            this.osRunner.copyPhotoToCodeUpDir(this.lastAddFile);
                            log.info("File copied to CodeUp home directory!");

                            log.info("Starting AppletViewer frame window...");
                            this.osRunner.startApp();
                            log.info("AppletViewer frame window started!");

                            /*
                            if (this.thFrame != null && frNokiaTrends==null)
                            {
                                this.thFrame.freeAllResources();
                                this.thFrame.dispose();

                                //this.thFrame.dispose();
                                this.thFrame = null;
                            }
                            log.info("Starting frame window...");

                            // Starts frame
                            this.thFrame = new TheFrame(file.getAbsolutePath());
                            thFrame.validate();

                            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
                            Dimension frameSize  = thFrame.getSize();

                            if (frameSize.height > screenSize.height)
                            {
                                frameSize.height = screenSize.height;
                            }

                            if (frameSize.width > screenSize.width)
                            {
                                frameSize.width = screenSize.width;
                            }

                            thFrame.setLocation((screenSize.width - frameSize.width) / 2,
                                (screenSize.height - frameSize.height) / 2);
                            thFrame.setVisible(true);
                            */
                        }
                    }
                }

                try
                {
                    log.info("Sleeping during 1000 milliseconds (1 second)...");
                    Thread.sleep(BTDirectoryListener.ONE_SEC);
                }
                catch (InterruptedException exc)
                {
                    log.severe("Thread.sleep interrupted! - " + exc.getMessage());
                }
            }
        }
        catch (Exception exc)
        {
            System.out.println("Exception - " + exc.getMessage());
        }
    }

    public static void main(String[] args)
    {
        System.out.println("Nokia Trends' Bluetooth Directory Listener");

        BTDirectoryListener btServer = null;

        if (args.length > 0)
        {
            try
            {
                if (args.length == 4)
                {
                    btServer = new BTDirectoryListener(args[0], args[1], args[2], args[3]);
                }
                else if (args.length == 3)
                {
                    btServer = new BTDirectoryListener(args[0], args[1], null, args[2]);
                }
                else if (args.length == 2)
                {
                    btServer = new BTDirectoryListener(args[0], args[1], null, null);
                }
                else
                {
                    btServer = new BTDirectoryListener(args[0], null, null, null);
                }

                // Initializing thread...
                Thread thBTServer = new Thread(btServer);
                thBTServer.start();
            }
            catch (BTDirectoryListenerException exc)
            {
                System.out.println("BTDirectoryListenerException - " + exc.getMessage());
            }
        }
        else
        {
            System.out.println("Invalid command line parameter number.");
        }
    }
}


// classe BTDirectoryListener
